<?php
/**
 * @package    JMyLife
 * @subpackage Components
 * @version    1.0.16
 * @copyright  2009-2010 Jeff Channell
 * administrator/components/com_jmylife/models/moderate.php
 * @license

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Moderate Model
 *
 * @package    JMyLife
 * @subpackage Components
 */
class JmylifeModelModerate extends JModel
{
	/**
	 * data
	 * @var stdClass
	 */
	var $_data = null;

	/**
	 * Items total - for pagination
	 * @var integer
	 */
	var $_total = null;

	/**
	 * Pagination object
	 * @var object
	 */
	var $_pagination = null;

	/**
	 * category list
	 * @var array
	 */
	var $_categories = null;

	/**
	 * version list
	 * @var array
	 */
	var $_versions = null;

	/**
	 * contructor
	 */
	function __construct() {
		parent::__construct();
		// J!1.6 doesn't use mainframe
		$appl =& JFactory::getApplication();
		global $option, $context;
		$view = JRequest::getWord('view');
		// Get pagination request variables
		$limit = $appl->getUserStateFromRequest('global.list.limit', 'limit', $appl->getCfg('list_limit'), 'int');
		$limitstart = JRequest::getInt('limitstart', 0);
		// In case limit has been changed, adjust it
		$limitstart = (0 == $limit ? 0 : (floor($limitstart / $limit) * $limit));
		// set in state
		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);
		// get filter order
		$filter_order      = $appl->getUserStateFromRequest($option.$view.'filter_order', 'filter_order', 's.`time`', 'cmd');
		$filter_order_Dir  = $appl->getUserStateFromRequest($option.$view.'filter_order_Dir', 'filter_order_Dir', 'DESC', 'word');
		// filter by category
		$filter_categoryid = $appl->getUserStateFromRequest($option.$view.'filter_categoryid', 'filter_categoryid', -1, 'int');
		// filter by search
		$search = $appl->getUserStateFromRequest($context.'search', 'search', '', 'string');
		$search = JString::strtolower($search);
		// set in state
		$this->setState('order', $filter_order);
		$this->setState('order_Dir', $filter_order_Dir);
		$this->setState('categoryid', $filter_categoryid);
		$this->setState('search', $search);
  }

  function _buildContentOrderBy() {
  	$orderby = ' ORDER BY '.$this->getState('order').' '.$this->getState('order_Dir').' ';
  	return $orderby;
	}

  function _buildContentWhere() {
  	$database   =& JFactory::getDBO();
  	$categoryid = (int)$this->getState('categoryid');
  	$search     = $this->getState('search');
  	$where = array();
  	if (0 < $categoryid) {
  		$where[] = ' s.`catid` = '.$categoryid.' ';
  	}
  	if (strlen($search)) {
  		$where[] = ' ( LOWER( s.`fulltext` ) LIKE '.$database->Quote('%'.$database->getEscaped($search, true).'%', false).' OR s.id = '.(int)$search.')';
		}
  	return $where;
	}

	/**
	 * builds item query
	 * @return string
	 *
	 */
	function _buildItemQuery() {
		$_JMLConfig = JmylifeConfiguration::getInstance();
		// initialize
		$orderby = '';
		$user =& JFactory::getUser();
		$cid = JRequest::getInt('cid');
		$fields  = array(
			's.*',
			'cat.`name` AS catname',
			'cat.`allow_comments` AS catcommentsallowed',
			'cat.`allow_votes` AS catvotesallowed'
		);
		$joins = array(
			'LEFT JOIN `#__jmylife_categories` AS cat ON ( cat.`id` = s.`catid` )',
		);
			$fields[] = 'COUNT( c.id ) AS numcomments';
			$joins[] = 'LEFT JOIN `#__jmylife_comments` AS c'
			           . ' ON ( c.`sid` = s.`id` AND c.`published` = 1 )';
		// build final query
		$query = 'SELECT '
		       . implode( ',', $fields ).' '
		       . 'FROM `#__jmylife_stories` AS s '
		       . implode( ' ', $joins ).' '
		       . 'WHERE s.`id` = '.$cid.' '
		       . 'GROUP BY s.`id`, cat.`id` '
		       ;
		// all done!
		return $query;
	}

	/**
	 * builds query
	 * @return string
	 *
	 */
	function _buildQuery()
	{
		$_JMLConfig = JmylifeConfiguration::getInstance();
		// initialize
		$orderby = '';
		$where   = $this->_buildContentWhere();
		$orderby = $this->_buildContentOrderBy();
		$fields  = array(
			's.*',
			'cat.`name` AS catname',
			'cat.`allow_comments` AS catcommentsallowed',
			'cat.`allow_votes` AS catvotesallowed'
		);
		$joins = array(
			'LEFT JOIN `#__jmylife_categories` AS cat ON ( cat.`id` = s.`catid` )',
		);
			$fields[] = 'COUNT( c.id ) AS numcomments';
			$joins[] = 'LEFT JOIN `#__jmylife_comments` AS c'
			           . ' ON ( c.`sid` = s.`id` AND c.`published` = 1 )';
		// build final query
		$query = 'SELECT '
		       . implode( ',', $fields ).' '
		       . 'FROM `#__jmylife_stories` AS s '
		       . implode( ' ', $joins ).' '
		       . ( count( $where ) > 0 ? 'WHERE '.implode(' AND ', $where ).' ' : '' )
		       . 'GROUP BY s.`id`, cat.`id` '
		       . $orderby
		       ;
		// all done!
		return $query;
	}

	/**
	 * builds category query
	 * @return string
	 *
	 */
	function _buildCategoryQuery()
	{
		$query = 'SELECT * FROM `#__jmylife_categories` WHERE `published` = 1';
		// all done!
		return $query;
	}

	/**
	 * gets list (override for long word handling)
	 * @return object list
	 * @since 1.0.5
	 */
	function _jmlGetList($query, $limitstart, $limit) {
		JLoader::import('components.com_jmylife.helpers.html', JPATH_ADMINISTRATOR, '');
		JLoader::import('components.com_jmylife.helpers.string', JPATH_ADMINISTRATOR, '');
		$html = JmylifeHtmlHelper::getInstance();
		$string = JmylifeStringHelper::getInstance();
		$_JMLConfig = JmylifeConfiguration::getInstance();
		// get original list
		$list = parent::_getList($query, $limitstart, $limit);
		for ($i=0; $i<count($list); $i++) {
			@$list[$i]->fulltext       = $html->clean(@$list[$i]->fulltext);
			@$list[$i]->admin_fulltext = $html->clean(@$list[$i]->admin_fulltext);
			@$list[$i]->user_alias     = $html->clean(@$list[$i]->user_alias);
			@$list[$i]->catname        = $html->clean(@$list[$i]->catname);
			@$list[$i]->title          = $html->clean(@$list[$i]->title);
			@$list[$i]->user_email     = $html->clean(@$list[$i]->user_email);
			if ((int)(@$list[$i]->allow_comments)) {
				@$list[$i]->comment_toggle = '<a href="javascript:void(0);" onclick="return listItemTask(\'cb'.$i.'\',\'unallowcomments\')" title="'.JText::_('COM_JMYLIFE_STORY_DISABLE_COMMENTS').'"><img src="images/tick.png" alt="'.JText::_('COM_JMYLIFE_STORY_COMMENTS_ALLOWED').'" border="0"></a>';
			} else {
				@$list[$i]->comment_toggle = '<a href="javascript:void(0);" onclick="return listItemTask(\'cb'.$i.'\',\'allowcomments\')" title="'.JText::_('COM_JMYLIFE_STORY_ENABLE_COMMENTS').'"><img src="images/publish_x.png" alt="'.JText::_('COM_JMYLIFE_STORY_COMMENTS_NOT_ALLOWED').'" border="0"></a>';
			}
			if ((int)(@$list[$i]->catcommentsallowed) == 0 || $_JMLConfig->getInt('allow_comments') == 0) {
				@$list[$i]->comment_toggle .= '<span class="error hasTip" title="'.JText::_('COM_JMYLIFE_WARN_COMMENTSDISABLED').'"><img border="0" alt="" src="'.JURI::root().'includes/js/ThemeOffice/warning.png"/></span>';
			}
			if ((int)(@$list[$i]->allow_votes)) {
				@$list[$i]->votes_toggle = '<a href="javascript:void(0);" onclick="return listItemTask(\'cb'.$i.'\',\'unallowvotes\')" title="'.JText::_('COM_JMYLIFE_STORY_DISABLE_VOTING').'"><img src="images/tick.png" alt="'.JText::_('COM_JMYLIFE_STORY_VOTING_ALLOWED').'" border="0"></a>';
			} else {
				@$list[$i]->votes_toggle = '<a href="javascript:void(0);" onclick="return listItemTask(\'cb'.$i.'\',\'allowvotes\')" title="'.JText::_('COM_JMYLIFE_STORY_ENABLE_VOTING').'"><img src="images/publish_x.png" alt="'.JText::_('COM_JMYLIFE_STORY_VOTING_NOT_ALLOWED').'" border="0"></a>';
			}
			if ((int)(@$list[$i]->catvotesallowed) == 0 || $_JMLConfig->getInt('allow_votes') == 0) {
				@$list[$i]->votes_toggle .= '<span class="error hasTip" title="'.JText::_('COM_JMYLIFE_WARN_VOTINGDISABLED').'"><img border="0" alt="" src="'.JURI::root().'includes/js/ThemeOffice/warning.png"/></span>';
			}
			if ($_JMLConfig->getInt('break_long_words') && JRequest::getWord('layout') != 'form') {
				@$list[$i]->fulltext   = $string->breakLongWords(@$list[$i]->fulltext);
				@$list[$i]->title      = $string->breakLongWords(@$list[$i]->title);
				@$list[$i]->user_alias = $string->breakLongWords(@$list[$i]->user_alias);
			}
			// since 1.0.11
			@$list[$i]->is_reported = @$list[$i]->reported == 1 ? '<img src="images/publish_y.png" alt="" />' : '';
		}
		return $list;
	}

	function getSearch() {
		return $this->getState('search');
	}

	/**
	 * Gets the list data
	 * @return stdClass object of listings
	 *
	 * @todo use Joomla Tables instead of simple queries
	 */
	function getData() {
		// if data hasn't already been obtained, load it
		if (empty($this->_data)) {
			$query = $this->_buildQuery();
			$this->_data = $this->_jmlGetList($query, $this->getState('limitstart'), $this->getState('limit'));
		}
		return $this->_data;
	}

	/**
	 * Gets the item data
	 * @return stdClass object of listings
	 *
	 * @todo use Joomla Tables instead of simple queries
	 */
	function getItem() {
		// if data hasn't already been obtained, load it
		if (empty($this->_data)) {
			$query = $this->_buildItemQuery();
			$this->_data = $this->_jmlGetList($query, $this->getState('limitstart'), $this->getState('limit'));
		}
		return $this->_data;
	}

	/**
	 * Gets the total amount of listings
	 * @return int
	 */
	function getTotal() {
		// Load the content if it doesn't already exist
		if (empty($this->_total)) {
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);
		}
		return $this->_total;
	}

	/**
	 * Gets the categories
	 * @return stdClass object of listings
	 *
	 * @todo use Joomla Tables instead of simple queries
	 */
	function getCategories() {
		$db = JFactory::getDBO();
		if (empty($this->_categories)) {
			$query = $this->_buildCategoryQuery();
			$db->setQuery($query);
			$this->_categories = $db->loadObjectList();
		}
		return $this->_categories;
	}

	/**
	 * Gets the listings pagination
	 * @return JPagination object
	 */
	function getPagination() {
		// Load the content if it doesn't already exist
		if (empty($this->_pagination)) {
			jimport('joomla.html.pagination');
			$this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit'));
		}
		return $this->_pagination;
	}

	/**
	 * gets any previous versions
	 * @return  array of stdClass objects
	 */
	function getVersions() {
		if (empty($this->_versions)) {
			$cid = JRequest::getInt('cid');
			$db = JFactory::getDBO();
			$db->setQuery("SELECT v.*, u.username AS editor FROM #__jmylife_stories_versions AS v LEFT JOIN #__users AS u ON (u.`id` = v.`edited_by`) WHERE v.`sid` = {$cid} GROUP BY v.`id` ORDER BY v.`id` DESC");
			$this->_versions = $db->loadObjectList();
		}
		return $this->_versions;
	}
}